@@ -1,6 +1,6 @@
 /*
    This file is part of the Eagle haplotype phasing software package
-   developed by Po-Ru Loh.  Copyright (C) 2015-2016 Harvard University.
+   developed by Po-Ru Loh.  Copyright (C) 2015-2018 Harvard University.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -191,7 +191,7 @@ namespace EAGLE {
 		  int argc, char**argv) const;
     void writeVcfNonRef(const std::string &vcfFile, const std::string &outFile, int inputChrom,
 			int chromX, double bpStart, double bpEnd, const std::string &writeMode,
-			bool keepMissingPloidyX, int argc, char **argv) const;
+			bool noImpMissing, bool keepMissingPloidyX, int argc, char **argv) const;
     void makeHardCalls(uint64 n0start, uint64 n0end, uint seed);
     void initRefIter(int refIter);
     void buildHashTables(int iter, int batch, int seed);
