@@ -1,6 +1,6 @@
 /*
    This file is part of the Eagle haplotype phasing software package
-   developed by Po-Ru Loh.  Copyright (C) 2015-2016 Harvard University.
+   developed by Po-Ru Loh.  Copyright (C) 2015-2018 Harvard University.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -237,10 +237,6 @@ namespace EAGLE {
 
       if (vm.count("vcfRef") || vm.count("vcfTarget") || vm.count("vcf")) { // VCF mode
 	if (vm.count("vcf")) { // non-ref mode
-	  if (noImpMissing) {
-	    cerr << "ERROR: --noImpMissing is only supported in ref-mode" << endl;
-	    return false;
-	  }
 	  if (bpFlanking != 0) {
 	    cerr << "ERROR: --bpFlanking is only supported in ref-mode" << endl;
 	    return false;
@@ -255,13 +251,14 @@ namespace EAGLE {
 	    cerr << "ERROR: --vcfTarget must be specified in reference-based phasing mode" << endl;
 	    return false;
 	  }
-	  if (pbwtIters > 1 && noImpMissing) {
-	    cerr << "ERROR: --pbwtIters cannot be greater than 1 if --noImpMissing is set" << endl;
-	    return false;
-	  }
 	  if (vcfRef.substr(vcfRef.length()-4) != string(".bcf")) {
 	    cerr << "WARNING: --vcfRef does not end in '.bcf'; BCF input is fastest" << endl;
 	  }
+	  if (chromStr=="0" && (bpStart>0 || bpEnd<1e9)) {
+	    cerr << "ERROR: --chrom is required when specifying --bpStart or --bpEnd in ref mode"
+		 << endl;
+	    return false;
+	  }
 	}
 
 	// vcf input checks for both ref and non-ref mode
@@ -291,7 +288,7 @@ namespace EAGLE {
 	  return false;
 	}
       }
-      else { // non-ref mode
+      else { // non-ref mode; plink input
 	if (famFile.empty()) {
 	  cerr << "ERROR: fam file must be specified either using --fam or --bfile"
 	       << endl;
@@ -308,7 +305,7 @@ namespace EAGLE {
 	  return false;
 	}
 	if (noImpMissing) {
-	  cerr << "ERROR: --noImpMissing is only supported in ref-mode" << endl;
+	  cerr << "ERROR: --noImpMissing is only supported with vcf input" << endl;
 	  return false;
 	}
 	if (bpFlanking != 0) {
